@@ -206,6 +206,7 @@ int SFont_Input2(SDL_Surface * Dest, SFont_FontInfo * Font, int x, int y,
   int ofs = (text[0] - 33) * 2 + 1;
   int leftshift;
   int ch = 0; //Fixes a warning: may be used unintialised.
+  char *sptr;
 
   if(ofs < 0) {
     leftshift = 0;
@@ -242,7 +243,9 @@ int SFont_Input2(SDL_Surface * Dest, SFont_FontInfo * Font, int x, int y,
 	text[strlen(text) - 1] = '\0';
       }
     } else {
-      sprintf(text, "%s%c", text, ch);
+      for(sptr = text; *sptr; ++sptr) ; //Find end of string.
+      *sptr++ = ch;
+      *sptr++ = '\0';
     }
 
     // If the new character would exceed the allowed width
