@@ -5,7 +5,7 @@
 #include "datafun.h"
 #include "globals.h"
 
-sdl_surfaces_t *surf_blubats; //!< blubat animation sequence
+extern sdl_surfaces_t *surf_blubats; //!< blubat animation sequence
 
 /*! \brief Initialise blubats
  *
