@@ -8,10 +8,10 @@
 void dead_pause() {
   play_tune(1);
   initialshield = 150;
-  xship = 10;
-  yship = ysize / 2;
-  xvel = 2;
-  yvel = 0;
+  shipdata.xship = 10;
+  shipdata.yship = ysize / 2;
+  shipdata.xvel = 2;
+  shipdata.yvel = 0;
   shieldlevel = 3 * W;
   laserlevel = 3 * W;
 }
@@ -94,10 +94,10 @@ void pausegame() {
 void reset_ship_state() {
   nships = 4;
   clear_score();
-  xvel = 0;
-  yvel = 0;
-  xship = 10;
-  yship = ysize / 2;
+  shipdata.xvel = 0;
+  shipdata.yvel = 0;
+  shipdata.xship = 10;
+  shipdata.yship = ysize / 2;
   shieldlevel = 3 * W;
   laserlevel = 3 * W;
   initialshield = 150;
