@@ -7,11 +7,11 @@
 /*! \brief Structure describing a Greeble
  */
 struct greeble {
-  int active;
-  float x, y;			// When landed, these represent an offset from the host rock
-  int target_rock_number;	// no longer used?. Should use rock_struct.greeb
-  int landed;
-  int boredom;			// Goes up while landed
+  int active;			//!< This is set nonzero if the greeblie is active (drawn, moving).
+  float x, y;			//!< When landed, these represent an offset from the host rock other coordinates on the screen.
+  int target_rock_number;	//!< The number of the rock the greeblie is flying to. Cross reference with use rock_struct.greeb.
+  int landed;			//!< Has the greeblie landed?
+  int boredom;			//!< Goes up while landed.
   /*! \brief Counter to the displayed image of the greeble.
    *
    * Every frame we increment this counter and divide it by three the
