@@ -64,9 +64,12 @@
 #define NULL_ERROR(a) COND_ERROR((a)==NULL)
 #define NULL_GURU(a, b, c) if((a)==NULL){initerror=strdup(SDL_GetError()); guru_meditation(GM_FLAGS_DEADEND, (b), (void*)(c)); return 1;}
 
+typedef struct Game_Over_Structure {
+  SDL_Surface *surf_t_game;	// Title element "game"
+  SDL_Surface *surf_t_over;	// Title element "over"
+} GameOver_t;
+
 // SDL_Surface global variables
-SDL_Surface *surf_t_game;	// Title element "game"
-SDL_Surface *surf_t_over;	// Title element "over"
 SDL_Surface *surf_t_paused;	// Title element "paused"
 SDL_Surface *surf_font_big;	// The big font
 SDL_Surface *surf_gauge_shield[2];	// The shield gauge 0=on,1=off
@@ -83,11 +86,7 @@ char topline[128];
 char *initerror = "";
 char *rockiff_name = NULL; //!< filename of rockdodger binary (iff) file with parameters/manipulation/configuration contents
 
-float rockrate, rockspeed;
-
-float countdown = 0;
 int laser = 0;
-int shieldsup = 0;
 int oss_sound_flag = 0;
 float faderate;
 
@@ -97,7 +96,7 @@ struct min_max {
 int rockfree_space_num = 0;
 int rockfree_space_max = 0;
 
-char *sequence[] = {
+static const char *sequence[] = {
   "Rock Dodger",
   "Press SPACE to start!",
   "http://bitbucket.org/rpkrawczyk/rockdodger/",
@@ -148,8 +147,7 @@ unsigned short screenshot_number = 0; /*!< Count the number of screen
  * \return 0 = OK, 1 = ERROR
  */
 int restart_game() {
-  int i;
-  const char *txts[] = {
+  static const char *txts[] = {
     "Congratulations! You have a high score.",
     "We have a winner!",
     "Wot a hero!",
@@ -179,9 +177,6 @@ int restart_game() {
 						7.28,
 						surf_screen->w));
 
-  for(i = 0; i < MAX_ROCKS; i++)
-    rock[i].active = 0;
-  rockspeed = 5.0 * xsize / 640;
   deactivate_greeblies();
   return 0;
 }
@@ -207,8 +202,8 @@ void drawlaser() {
   for(i = 0; i < MAX_ROCKS; i++) {
     if(rock[i].active) {
       SDL_Surface *rocksurf = get_rock_surface(i, 0);
-      if(yship + 12 > rock[i].y && yship + 12 < rock[i].y + rocksurf->h
-	 && xship + 32 < rock[i].x + (rocksurf->w / 2)
+      if(shipdata.yship + 12 > rock[i].y && shipdata.yship + 12 < rock[i].y + rocksurf->h
+	 && shipdata.xship + 32 < rock[i].x + (rocksurf->w / 2)
 	 && rock[i].x + (rocksurf->w / 2) < xc) {
 	xc = rock[i].x + (rocksurf->w / 2);
 	hitrock = i;
@@ -228,8 +223,8 @@ void drawlaser() {
 	greebx += rock[g->target_rock_number].x;
 	greeby += rock[g->target_rock_number].y;
       }
-      if(yship + 12 > greeby && yship + 12 < greeby + greebheight &&
-	 xship + 32 < greebx + (greebwidth / 2) && greebx + (greebwidth / 2) <
+      if(shipdata.yship + 12 > greeby && shipdata.yship + 12 < greeby + greebheight &&
+	 shipdata.xship + 32 < greebx + (greebwidth / 2) && greebx + (greebwidth / 2) <
 	 xc) {
 	xc = greebx + (greebwidth / 2);
 	hitgreeb = i;
@@ -241,15 +236,15 @@ void drawlaser() {
     heat_rock_up(hitrock, movementrate);
   if(hitgreeb >= 0)
     kill_greeb(hitgreeb);
-  if((yptr = check_yellifish_hit(yellifishs, xship + 32, xc, yship + 12))) {
-    inc_score(xship + 32, yship + 12, YELLIFISH_KILL_POINTS);
+  if((yptr = check_yellifish_hit(yellifishs, shipdata.xship + 32, xc, shipdata.yship + 12))) {
+    inc_score(shipdata.xship + 32, shipdata.yship + 12, YELLIFISH_KILL_POINTS);
     kill_yellifish(yptr);
   }
-  if((hitgreeb = check_blubat_hit((int) yship + 12, xc)) >= 0) {
+  if((hitgreeb = check_blubat_hit((int) shipdata.yship + 12, xc)) >= 0) {
     kill_blubat(hitgreeb);
   }
-  // Plot a number of random dots between xship and xsize (or xc as in x-collision)
-  draw_random_dots(surf_screen, yship + 12, xship + 32, xc);
+  // Plot a number of random dots between shipdata.xship and xsize (or xc as in x-collision)
+  draw_random_dots(surf_screen, shipdata.yship + 12, shipdata.xship + 32, xc);
 }
 
 /*! \brief Set the SDL video mode
@@ -271,7 +266,7 @@ SDL_Surface *set_video_mode() {
 }
 
 
-int init() {
+int init(GameOver_t *game_over_data) {
   int i;
 
   // Make sure that the name_input_buf is null terminated.
@@ -361,8 +356,8 @@ int init() {
   // GM_SS_Graphics | GM_GE_IOError
   NULL_GURU(surf_t_rock = load_image("rock.png", 255, 0, 0), GM_SS_Graphics | GM_GE_IOError, 0x524f434b);
   NULL_GURU(surf_t_dodger = load_image("dodgers.png", 255, 0, 0), GM_SS_Graphics | GM_GE_IOError, 0x444f4447);
-  NULL_GURU(surf_t_game = load_image("game.png", 255, 0, 0), GM_SS_Graphics | GM_GE_IOError, 0x47414d45);
-  NULL_GURU(surf_t_over = load_image("over.png", 255, 0, 0), GM_SS_Graphics | GM_GE_IOError, 0x4f564552);
+  NULL_GURU(game_over_data->surf_t_game = load_image("game.png", 255, 0, 0), GM_SS_Graphics | GM_GE_IOError, 0x47414d45);
+  NULL_GURU(game_over_data->surf_t_over = load_image("over.png", 255, 0, 0), GM_SS_Graphics | GM_GE_IOError, 0x4f564552);
   NULL_GURU(surf_t_paused = IMG_Load(load_file("paused.png")), GM_SS_Graphics | GM_GE_IOError, 0x50415553);
   NULL_GURU(surf_gauge_laser[0] = IMG_Load(load_file("laser0.png")), GM_SS_Graphics | GM_GE_IOError, 0x4c415330);
   NULL_GURU(surf_gauge_laser[1] = IMG_Load(load_file("laser1.png")), GM_SS_Graphics | GM_GE_IOError, 0x4c415331);
@@ -427,7 +422,7 @@ void showgauge(int x, SDL_Surface * surf[2], float fraction) {
 
 void display_version_n_text() {
   char buf[120];
-  char *text;
+  const char *text;
 
   assert(edition_name);
   sprintf(buf, "Version %s - The %s edition  ( %s )", VERSION, edition_name, COMPILEDATE);
@@ -465,7 +460,16 @@ void display_highscores() {
  * mode.
  */
 void enter_highscore(void) {
+  int i;
+
   assert(state == HIGH_SCORE_ENTRY);
+  for(i = 0; i < MAXIMUM_HIGH_SCORE_ENTRIES - 1; ++i) {
+    if(high[i].score < high[i + 1].score) {
+      i = guru_meditation(GM_FLAGS_ABORTIFY | GM_FLAGS_DEADEND, GM_SS_GamePort | GM_GE_BadParm | GURU_SEC_main, &enter_highscore);
+      printf("guru_meditation() = %d\n", i);
+      return;
+    }
+  }
   if(scorerank >= 0) {
     play_tune(2);
     if(SFont_Input
@@ -502,45 +506,53 @@ void draw_titlepage(void) {
   display_version_n_text();
 }
 
-
-void draw_gameover(void) {
+/*! \brief Draw the fading game over.
+ *
+ * This function draws the gostly GAME OVER once the last ship was
+ * lost. Currently these are also global variables but this is goint
+ * to change.
+ *
+ * \param fadetimer timer variable
+ * \param gost game over structure pointer
+ * \param surf_screen target-screen surface
+ */
+void draw_gameover(float fadetimer, const GameOver_t *gost, SDL_Surface *surf_screen) {
   SDL_Rect src, dest;
   float fadegame, fadeover;
 
-  if(fadetimer < 3.0 / faderate)
+  if(fadetimer < 3.0 / faderate) {
     fadegame = fadetimer / (3.0 / faderate);
-  else
+  } else {
     fadegame = 1.0;
-
-  if(fadetimer < 3.0 / faderate)
+  }
+  if(fadetimer < 3.0 / faderate) {
     fadeover = 0.0;
-  else if(fadetimer < 6.0 / faderate)
+  } else if(fadetimer < 6.0 / faderate) {
     fadeover = ((3.0 / faderate) - fadetimer) / (6.0 / faderate);
-  else
+  } else {
     fadeover = 1.0;
-
-  src.w = surf_t_game->w;
-  src.h = surf_t_game->h;
+  }
+  src.w = gost->surf_t_game->w;
+  src.h = gost->surf_t_game->h;
   src.x = 0;
   src.y = 0;
   dest.w = src.w;
   dest.h = src.h;
   dest.x = (xsize - src.w) / 2;
   dest.y = (ysize - src.h) / 2 - 40;
-  SDL_SetAlpha(surf_t_game, SDL_SRCALPHA,
+  SDL_SetAlpha(gost->surf_t_game, SDL_SRCALPHA,
 	       (int) (fadegame *
-		      (200 + 55 * cosf(fadetimer += movementrate / 1.0))));
-  SDL_BlitSurface(surf_t_game, &src, surf_screen, &dest);
-
-  src.w = surf_t_over->w;
-  src.h = surf_t_over->h;
+		      (200 + 55 * cosf(fadetimer))));
+  SDL_BlitSurface(gost->surf_t_game, &src, surf_screen, &dest);
+  src.w = gost->surf_t_over->w;
+  src.h = gost->surf_t_over->h;
   dest.w = src.w;
   dest.h = src.h;
   dest.x = (xsize - src.w) / 2;
   dest.y = (ysize - src.h) / 2 + 40;
-  SDL_SetAlpha(surf_t_over, SDL_SRCALPHA,
+  SDL_SetAlpha(gost->surf_t_over, SDL_SRCALPHA,
 	       (int) (fadeover * (200 + 55 * sinf(fadetimer))));
-  SDL_BlitSurface(surf_t_over, &src, surf_screen, &dest);
+  SDL_BlitSurface(gost->surf_t_over, &src, surf_screen, &dest);
 }
 
 
@@ -580,13 +592,78 @@ void display_infoscreen() {
 }
 
 
+/*! \brief Handle powerups
+ *
+ * This function checks if there is a powerup displayed, if the ship
+ * has "hit" it, and than handles the powerup procedure.
+ *
+ * \param shieldsup is the shield up? 
+ */
+void handle_powerups(int shieldsup) {
+  // If the game displays the laserpowerup. The check is done
+  // explicitly.
+  if(get_current_powerup() != POWERUP_NONE) {
+    display_powerup(surf_screen);
+    //Yes, shields protect against *everything*!
+    if(!((initialshield > 0) || (shieldsup && shieldlevel > 0))) {
+      int bangpowerup = ship_check_collision(surf_screen);
+      if(bangpowerup) {
+	switch(get_current_powerup()) {
+	case POWERUP_LASER:
+	  laserlevel += 16;
+	  if(laserlevel > 3 * W) {
+	    laserlevel = 3 * W;
+	  }
+	  break;
+	case POWERUP_SHIELD:
+	  shieldlevel += 13;
+	  if(shieldlevel > 3 * W) {
+	    shieldlevel = 3 * W;
+	  }
+	  break;
+	case POWERUP_LIFE:
+	  nships += 1; //Increase number of ships.
+	  break;
+	  /* None will never happen as it is checked above. But better
+	     to be save then sorry... */
+	case POWERUP_NONE:
+	case POWERUP_MAXPOWERUP:
+	  puts("Illegal powerup!");
+	  play_sound(4);
+	  /*
+	   * We use this goto so that only one sound is played. The
+	   * powerup must always be deactivated.
+	   */
+	  goto illegal_powerup_encounterd;
+	  break;
+	}
+	play_sound(5);
+      illegal_powerup_encounterd:
+	deactivate_powerup();
+      }
+    }
+  }
+}
+
+static void draw_paused(SDL_Surface *screen) {
+  SDL_Rect dest;
+  dest.w = surf_t_paused->w;
+  dest.h = surf_t_paused->h;
+  dest.x = (xsize - dest.w) / 2;
+  dest.y = (ysize - dest.h) / 2;
+  SDL_BlitSurface(surf_t_paused, NULL, screen, &dest);
+}
+
+
 /*! \brief Draw all
  *
  * Draw all the objects on the standard surface.
  *
+ * \param shieldsup is the shield up?
+ * \param game_over_data pointer to game over structure
  * \return bang=0 if everything ok, bang=1 if ship was hit.
  */ 
-int draw() {
+int draw(int shieldsup, const GameOver_t *game_over_data) {
   int i;
   int bang;
 
@@ -604,6 +681,12 @@ int draw() {
   // Draw ship
   if(state == GAME_PLAY || state == GAME_PAUSED || state == DEMO) {
     draw_ship(surf_screen);
+    // Create more engine dots comin out da back
+    if(state == GAME_PAUSED) {
+      create_engine_dots(87);
+    } else {
+      create_engine_dots(250);
+    }
   }
 
   // Draw all the rocks, in all states. 
@@ -616,7 +699,8 @@ int draw() {
   switch (state) {
   case GAME_OVER:
     // If it's game over, show the game over graphic in the dead centre
-    draw_gameover();
+    draw_gameover(fadetimer, game_over_data, surf_screen);
+    fadetimer += movementrate / 1.0;
     break;
   case RESTART_GAME:
   case TITLE_PAGE:
@@ -636,17 +720,12 @@ int draw() {
     display_version_n_text();
     break;
   case GAME_PAUSED:
-    {
-      SDL_Rect dest;
-      dest.w = surf_t_paused->w;
-      dest.h = surf_t_paused->h;
-      dest.x = (xsize - dest.w) / 2;
-      dest.y = (ysize - dest.h) / 2;
-      SDL_BlitSurface(surf_t_paused, NULL, surf_screen, &dest);
-    }
+    draw_paused(surf_screen);
     break;
   case DEAD_PAUSE:
   case GAME_PLAY:
+    // This many points per screen update.
+    inc_score(0, 0, SCORE_INCREMENT_FACTOR);
     break;
   case DEMO:
 #if defined(DEBUG)
@@ -677,39 +756,7 @@ int draw() {
       // on the ship are still black, and not covered up by rocks
       bang = ship_check_collision(surf_screen);
     }
-
-    //If in game display the laserpowerup
-    if(get_current_powerup()) {
-      display_powerup(surf_screen);
-      //Yes, shields protect against *everything*!
-      if(!((initialshield > 0) || (shieldsup && shieldlevel > 0))) {
-	int bangpowerup = ship_check_collision(surf_screen);
-	if(bangpowerup) {
-	  switch(get_current_powerup()) {
-	  case POWERUP_LASER:
-	    laserlevel += 16;
-	    if(laserlevel > 3 * W)
-	      laserlevel = 3 * W;
-	    break;
-	  case POWERUP_SHIELD:
-	    shieldlevel += 13;
-	    if(shieldlevel > 3 * W)
-	      shieldlevel = 3 * W;
-	    break;
-	  case POWERUP_LIFE:
-	    nships += 1; //Increase number of ships.
-	    break;
-	  case POWERUP_MAXPOWERUP:
-	  case POWERUP_NONE:
-	    puts("Illegal powerup!");
-	    play_sound(4);
-	    break;
-	  }
-	  deactivate_powerup();
-	  play_sound(5);
-	}
-      }
-    }
+    handle_powerups(shieldsup);
   }
 
   // Draw all the little ships
@@ -728,8 +775,6 @@ int draw() {
   PutString(surf_screen, 0, 0, tempbuf);
 #endif
 
-  if(state == GAME_PLAY)
-    inc_score(0, 0, SCORE_INCREMENT_FACTOR);	//This many points per screen update
   // Always draw the last score
   i = xsize - 250;
   snprintf(topline, sizeof(topline), "Score: %ld", score);
@@ -757,11 +802,11 @@ void crash_boom_bang() {
       state_timeout = 200.0;
       fadetimer = 0.0;
       faderate = movementrate;
-      level = floorf(level); /*Otherwise if level is >x.98 and
-			       smaller then x+1 the function
-			       draw_infinite_black() will cause
-			       problems. It will loop the sound and
-			       will not black the background.*/
+       /* Otherwise if level is >x.98 and smaller then x+1 the
+	* function draw_infinite_black() will cause problems. It will
+	* loop the sound and will not black the background.
+	*/
+      level = floorf(level);
       //Remove blubats
       deactivate_all_blubats();
       //Remove yellifish
@@ -793,16 +838,16 @@ Figure 1–1: A typical game loop
 
  */
 
-void ready_for_takeoff() {
-  int i;
-  for(i = 0; i < MAX_ROCKS; i++)
-    rock[i].active = 0;
+/*! \brief Prepare ship for start.
+ *
+ * This function is called before the game begins. It deactivates all
+ * enemies, resets the level, and sets the game-play state.
+ */
+void ready_for_takeoff(void) {
   deactivate_greeblies();
   deactivate_all_blubats();
   deactivate_all_yellifish(yellifishs);
-  rockspeed = 5.0 * xsize / 640.0;
-  rockrate = 0;
-  countdown = 1.111;
+  reset_rocks();
   level = 1;
   last_levelup_snd = 0;
   state = GAME_PLAY;
@@ -812,15 +857,133 @@ void ready_for_takeoff() {
   reset_ship_state();
 }
 
+/*! \brief perform the demo feat
+ *
+ * The ship is moved into the biggest available space between the
+ * rocks. In debug mode bars visualize the available spaces. Shields
+ * are avtivated if the rocks are too close.
+ *
+ * \param inputstate pointer the the input state
+ */
+void perform_demonstration(inputstate_t *inputstate) {
+  int i, j;
+  
+  rockfree_space_num = 1;
+  rockfree_space_mm[0].min = 0;
+  rockfree_space_mm[0].max = ysize;
+  for(i = 0; i < MAX_ROCKS; ++i) {
+    if(rock[i].active && rock[i].x + get_rock_surface(i, 0)->w > shipdata.xship) {
+      int y = rock[i].y;
+      int Y = rock[i].y + get_rock_surface(i, 0)->h;
+      for(j = 0; j < rockfree_space_num; ++j) {
+	if(y > rockfree_space_mm[j].min && Y < rockfree_space_mm[j].max) {
+	  rockfree_space_mm[rockfree_space_num] = rockfree_space_mm[j];
+	  rockfree_space_mm[j].max = y;
+	  rockfree_space_mm[rockfree_space_num].min = Y;
+	  ++rockfree_space_num;
+	  break;
+	} else if(y > rockfree_space_mm[j].min && y < rockfree_space_mm[j].max) {
+	  rockfree_space_mm[j].max = y;
+	  break;
+	} else if(Y > rockfree_space_mm[j].min && Y < rockfree_space_mm[j].max) {
+	  rockfree_space_mm[j].min = Y;
+	}
+	if((rockfree_space_mm[j].max - rockfree_space_mm[j].min) < 10 && rockfree_space_num > 0) {
+	  rockfree_space_mm[j] = rockfree_space_mm[rockfree_space_num--];
+	}
+      }
+    }
+  }
+  for(i = 0, j = 0; j < rockfree_space_num; ++j) {
+    if(rockfree_space_mm[j].max - rockfree_space_mm[j].min >= rockfree_space_mm[i].max - rockfree_space_mm[i].min) i = j;
+  }
+  rockfree_space_max = i;
+  inputstate->inputstate[SHIELD] = 1;
+  for(j = 0; j < rockfree_space_num; ++j) {
+    if(shipdata.yship >= rockfree_space_mm[j].min && shipdata.yship + surfaces_ship->surfaces[0]->h <= rockfree_space_mm[j].max) {
+      inputstate->inputstate[SHIELD] = 0;
+      break;
+    }
+  }
+  inputstate->inputstate[LASER] = inputstate->inputstate[SHIELD];
+  if(shipdata.yship < rockfree_space_mm[rockfree_space_max].min) {
+    inputstate->inputstate[DOWN] = 1;
+  }
+  if(shipdata.yship > rockfree_space_mm[rockfree_space_max].max) {
+    inputstate->inputstate[UP] = 1;
+  }
+}
+
+
+/*! \brief Move the enemy game objects
+ *
+ * The enemy object are moved. This function is called once per
+ * frame. Currently this includes the rocks, greeblies, blubats, and
+ * yellifish.
+ * 
+ * \param yelli Yellifish subsystem pointer
+ * \param preyx x-position of prey
+ * \param preyy y-position of prey
+ */
+void update_enemy_objects(yellifishsubsystem_t *yelli, float preyx, float preyy) {
+  assert(yelli != NULL);
+  // Move all the rocks.
+  update_rocks();
+  // And now the greeblies.
+  move_all_greeblies();
+  // Update blubats.
+  move_all_blubats();
+  // Update yellifishs
+  update_yellifishs(yelli, preyx, preyy);
+}
+
+
+/*! \brief Get/update the input state.
+ *
+ * This will get the newest input state, getting keyboard and joystick
+ * input.
+ *
+ * \param inputstate pointer to inputstate structure to be filled
+ * \return laserstatus 0=laser does not fire, 1=laser fires
+ */
+int get_input_state(inputstate_t *inputstate) {
+  int laser = 0;
+
+  if(state == GAME_PLAY || state == DEMO) {
+    if(state != DEMO) get_joystick_input(inputstate);
+    if(inputstate->inputstate[UP]) {
+      shipdata.yvel -= 1.5 * movementrate;
+      create_engine_dots2(5, 3); // Create engine dots out the side we're moving from
+    }
+    if(inputstate->inputstate[DOWN]) {
+      shipdata.yvel += 1.5 * movementrate;
+      create_engine_dots2(5, 1);
+    }
+    if(inputstate->inputstate[LEFT]) {
+      shipdata.xvel -= 1.5 * movementrate;
+      create_engine_dots2(5, 2);
+    }
+    if(inputstate->inputstate[RIGHT]) {
+      shipdata.xvel += 1.5 * movementrate;
+      create_engine_dots2(5, 0);
+    }
+    if(inputstate->inputstate[LASER]) {
+      laser = 1;
+    }
+  }
+  return laser;
+}
+
 
 /*! \brief game loop called once per frame
  *
  * This is the game loop where all the magic happens. This loop is
  * only left if state == QUIT_GAME or RESTART_GAME.
  *
+ * \param game_over_data pointer to game-over structure 
  * \return 0 == quit the game, 1 == restart game
  */
-int gameloop() {
+int gameloop(const GameOver_t *game_over_data) {
   inputstate_t inputstate;
   Uint8 *keystate;
 
@@ -828,82 +991,25 @@ int gameloop() {
     update_movement_rate();
     keystate = input_subsystem(&inputstate);
     handle_state_timeout(infoscreen_enabled);
-
-    if(state != GAME_PAUSED) {
-      rockspeed = 4.885 * xsize / 640.0 + sqrtf(floorf(level)) / 5.0;
-      rockrate = 0.13182 + sqrtf(floorf(level)) / 18.0;
-      if(level - (int) level < 0.8) {
-	if(state != GAME_PLAY && state != DEMO)
-	  countdown -= 0.3 * movementrate;
-	else
-	  countdown -= rockrate * movementrate;
-	while(countdown < 0) {
-	  countdown++;
-	  create_rock(rockspeed);
-	}
-      }
-    }
+    ingame_maybe_rock();
 
     switch(state) {
     case TITLE_PAGE:
       // Before the rocks are drawn, the greeblies are shown playing amongst them.
       activate_one_greeblie();
-      // Move all the rocks
-      update_rocks();
+      // TODO: where is the prey?
+      update_enemy_objects(yellifishs, 0, 0);
       // Move the powerup
       update_powerup();
-      move_all_greeblies();
-      move_all_blubats();
       update_sparkles(surf_screen);
       if((last_ticks / 5391) % 7 == 6) {
 	//Rockmania!
-	create_rock(rockspeed);
+	create_rock();
       }
       break;
     case DEMO:
-      { //TODO: put in own function/file (including AI).
-	int i, j;
-	rockfree_space_num = 1;
-	rockfree_space_mm[0].min = 0;
-	rockfree_space_mm[0].max = ysize;
-	for(i = 0; i < MAX_ROCKS; ++i) {
-	  if(rock[i].active && rock[i].x + get_rock_surface(i, 0)->w > xship) {
-	    int y = rock[i].y;
-	    int Y = rock[i].y + get_rock_surface(i, 0)->h;
-	    for(j = 0; j < rockfree_space_num; ++j) {
-	      if(y > rockfree_space_mm[j].min && Y < rockfree_space_mm[j].max) {
-		rockfree_space_mm[rockfree_space_num] = rockfree_space_mm[j];
-		rockfree_space_mm[j].max = y;
-		rockfree_space_mm[rockfree_space_num].min = Y;
-		++rockfree_space_num;
-		break;
-	      } else if(y > rockfree_space_mm[j].min && y < rockfree_space_mm[j].max) {
-		rockfree_space_mm[j].max = y;
-		break;
-	      } else if(Y > rockfree_space_mm[j].min && Y < rockfree_space_mm[j].max) {
-		rockfree_space_mm[j].min = Y;
-	      }
-	      if((rockfree_space_mm[j].max - rockfree_space_mm[j].min) < 10 && rockfree_space_num > 0) {
-		rockfree_space_mm[j] = rockfree_space_mm[rockfree_space_num--];
-	      }
-	    }
-	  }
-	}
-	for(i = 0, j = 0; j < rockfree_space_num; ++j) {
-	  if(rockfree_space_mm[j].max - rockfree_space_mm[j].min >= rockfree_space_mm[i].max - rockfree_space_mm[i].min) i = j;
-	}
-	rockfree_space_max = i;
-	inputstate.inputstate[SHIELD] = 1;
-	for(j = 0; j < rockfree_space_num; ++j) {
-	  if(yship >= rockfree_space_mm[j].min && yship + surfaces_ship->surfaces[0]->h <= rockfree_space_mm[j].max) {
-	    inputstate.inputstate[SHIELD] = 0;
-	    break;
-	  }
-	}
-	inputstate.inputstate[LASER] = inputstate.inputstate[SHIELD];
-	if(yship < rockfree_space_mm[rockfree_space_max].min) inputstate.inputstate[DOWN] = 1;
-	if(yship > rockfree_space_mm[rockfree_space_max].max) inputstate.inputstate[UP] = 1;
-      }
+      perform_demonstration(&inputstate);
+      //Fall through
     case DEAD_PAUSE:
     case GAME_PLAY:
       // Increase difficulty
@@ -917,17 +1023,11 @@ int gameloop() {
       if(rnd() < .02 * level * movementrate) {
 	activate_one_greeblie();
       }
-      // Move all the rocks
-      update_rocks();
+      update_enemy_objects(yellifishs, shipdata.xship, shipdata.yship - 87.8228228);
       // Move ship to new position
       ship_update_position();
       // Move the powerup
       update_powerup();
-      move_all_greeblies();
-      //Update blubats
-      move_all_blubats();
-      //Update yellifishs
-      update_yellifishs(yellifishs, xship, yship - 87.8228228);
       update_sparkles(surf_screen);
       break;
     case GAME_OVER:
@@ -936,16 +1036,11 @@ int gameloop() {
     case INFO_SCREEN:
     case HIGH_SCORE_DISPLAY:
     case HIGH_SCORE_ENTRY:
-      // Move all the rocks
-      update_rocks();
+      update_enemy_objects(yellifishs, xsize / 2, ysize);
       // Move ship to new position
       ship_update_position();
       // Move the powerup
       update_powerup();
-      move_all_greeblies();
-      //Update blubats
-      move_all_blubats();
-      update_yellifishs(yellifishs, xsize, 0);
       update_sparkles(surf_screen);
       break;
     case SETUP:
@@ -964,7 +1059,9 @@ int gameloop() {
     }
 
     //Was the ship hit?
-    if(draw() && (state == GAME_PLAY || state == DEMO)) crash_boom_bang();
+    if(draw(inputstate.inputstate[SHIELD], game_over_data) && (state == GAME_PLAY || state == DEMO)) {
+      crash_boom_bang();
+    }
 
     if(keystate[SDLK_SPACE]
        && (state == HIGH_SCORE_DISPLAY || state == TITLE_PAGE
@@ -972,32 +1069,7 @@ int gameloop() {
       ready_for_takeoff();
     }
 
-    laser = 0;
-    if(state == GAME_PLAY || state == DEMO) {
-      if(state != DEMO) get_joystick_input(&inputstate);
-      if(inputstate.inputstate[UP]) {
-	yvel -= 1.5 * movementrate;
-	create_engine_dots2(5, 3); // Create engine dots out the side we're moving from
-      }
-      if(inputstate.inputstate[DOWN]) {
-	yvel += 1.5 * movementrate;
-	create_engine_dots2(5, 1);
-      }
-      if(inputstate.inputstate[LEFT]) {
-	xvel -= 1.5 * movementrate;
-	create_engine_dots2(5, 2);
-      }
-      if(inputstate.inputstate[RIGHT]) {
-	xvel += 1.5 * movementrate;
-	create_engine_dots2(5, 0);
-      }
-      // Create more engine dots comin out da back
-      create_engine_dots(250);
-      if(inputstate.inputstate[LASER]) {
-	laser = 1;
-      }
-      shieldsup = inputstate.inputstate[SHIELD];
-    }
+    laser = get_input_state(&inputstate);
     //Check for screen shot.
     if(state != HIGH_SCORE_ENTRY && inputstate.inputstate[SCREENSHOT]) {
       int ret;
@@ -1032,6 +1104,7 @@ void shutdown_subsystems() {
 int main(int argc, char **argv) {
   int i, x;
   int force_intro = 0;
+  GameOver_t game_over_data;
 
   assert(RD_VIDEO_BPP == 8 * sizeof(RD_VIDEO_TYPE));
   oss_sound_flag = 1;
@@ -1082,7 +1155,7 @@ int main(int argc, char **argv) {
     }
 
   initrnd();
-  if(init()) {
+  if(init(&game_over_data)) {
     printf("Cannot start: '%s'\n", initerror);
     return 1;
   }
@@ -1097,8 +1170,10 @@ int main(int argc, char **argv) {
     temporary_disable_key_input();
     if(oss_sound_flag) play_tune(0);
     while(1) {
-      if(restart_game() != 0) break; //We are gone now...
-      i = gameloop();
+      if(restart_game() != 0) {
+	break; //We are gone now...
+      }
+      i = gameloop(&game_over_data);
       if(i == 0) {
 	break;
       } else if(i == 1) {
