@@ -13,7 +13,7 @@ static struct PowerupInformationForIni {
   { "NONE", NULL },
   { "Laserpowerups", "laserpowerup.%02X.png", 0, 0, 255 },
   { "Shieldpowerups", "shieldpowerup.%02X.png", 0, 0, 255 },
-  { "Lifepowerups", "lifepowerup.%02X.ilbm", 0, 255, 0 }
+  { "Lifepowerups", "lifepowerup.%02X.png", 0, 255, 0 }
 };
 
 /*! \brief array with power surfaces
