@@ -9,6 +9,10 @@
 #include <assert.h>
 #include <math.h>
 
+float rockrate; //!< How many rocks are created per time unit?
+float rockspeed; //!< How fast are the rocks?
+float countdown = 0; //!< countdown before next rock creation
+
 struct rock_struct rock[MAX_ROCKS];
 
 sdl_surfaces_t *surf_rocks = NULL; //!< All the rocky asteroid surfaces.
@@ -38,7 +42,9 @@ SDL_Surface *get_rock_surface(int rockidx, int hot) {
   }
   if(imgidx >= surfs->num_surfaces) {
     Uint32 addr = rockidx;
-    if(hot) addr |= 0x8000;
+    if(hot) {
+      addr |= 0x8000;
+    }
     addr |= (imgidx & 0xFF) << 24;
     addr |= (surfs->num_surfaces & 0xFF) << 16;
     guru_meditation(GM_FLAGS_RECOVERY | GM_FLAGS_CHOICE | GM_FLAGS_ABORTIFY, GM_SS_Misc | GM_GE_BadParm | GURU_SEC_rocks, (void *)(unsigned long)addr);
@@ -49,22 +55,27 @@ SDL_Surface *get_rock_surface(int rockidx, int hot) {
 
 
 void heat_rock_up(int rockidx, float mvmt) {
+  struct rock_struct *rptr;
+  float plume_phi;
+  
   if(rockidx < 0 || rockidx >= MAX_ROCKS) {
     guru_meditation(GM_FLAGS_DEADEND, GM_SS_Misc | GM_GE_BadParm | GURU_SEC_rocks, (void *)(long)rockidx);
   }
-  struct rock_struct *rptr = rock + rockidx;
+  rptr = rock + rockidx;
   rock[rockidx].heat += mvmt / rock[rockidx].heat_capacity;
   if(rock[rockidx].type_class == 2) {
     SDL_Surface *image = get_rock_surface(rockidx, 0);
     float size = (image->w + image->h) / 2.0;
     create_plumedots(9, rptr->x + image->w / 2, rptr->y + image->h / 2, rptr->plume_angle, 2, 17.27);
-    rptr->xvel += 1.442 * cosf(rptr->plume_angle) / size;
-    rptr->yvel += 1.442 * sinf(rptr->plume_angle) / size;
+    plume_phi = (rptr->plume_angle + 180) / 360.0 * 2 * M_PI;
+    rptr->xvel += 1.442 * cosf(plume_phi) / size;
+    rptr->yvel += 1.442 * -sinf(plume_phi) / size;
   }
 }
 
 
-void create_rock(float rockspeed) {
+void create_rock() {
+  // rockspeed current default speed of the rock, modified by a random amount
   struct rock_struct *rockptr;
 
   for(rockptr = rock; rockptr < rock + MAX_ROCKS; ++rockptr) {
@@ -99,6 +110,48 @@ void create_rock(float rockspeed) {
 #endif
 }
 
+static void way_of_the_exploding_rock(struct rock_struct *rockp, int last_rock, SDL_Surface *image, SDL_Surface *surf_screen) {
+  int g;
+  short x, y;
+
+  rockp->active = 0;
+  play_sound(1 + (int) (rnd() * 3));
+  switch(rockp->type_class) {
+  case 2: //Ice rocks are different
+    x = rockp->x + image->w / 2;
+    y = rockp->y + image->h / 2;
+    create_plumedots(1659, x, y, 180, 360, 10.9312);
+    create_plumedots(539, x, y, 180, 360, 2.73002);
+    create_plumedots(367, x, y, 90, 109, 4.656);
+    create_plumedots(367, x, y, 270, 109, 4.656);
+    create_plumedots(467, x + rockp->xvel, y + rockp->yvel, 90, 109, 1.7656);
+    create_plumedots(467, x + rockp->xvel, y + rockp->yvel, 270, 109, 1.7656);
+    create_plumedots(426, x - rockp->xvel, y - rockp->yvel, 90, 109, 1.37656);
+    create_plumedots(426, x - rockp->xvel, y - rockp->yvel, 270, 109, 1.37656);
+    break;
+  default:
+    makebangdots(rockp->x, rockp->y, rockp->xvel, rockp->yvel,
+		 image, 10, 3);
+  }
+
+  // If a greeblie was going to this rock
+  if((g = rockp->greeb)) {
+    // ...and had landed, then the greeblie explodes too.
+    if(the_greeblies[g].landed)
+      kill_greeb(g);
+    else {
+      // If the greeblie is not yet landed, then it must now
+      // find another rock to jump to. Choose the last active
+      // rock found.
+      rockp->greeb = 0;
+      rock[last_rock].greeb = g;
+      the_greeblies[g].target_rock_number = last_rock;
+    }
+  }
+#ifdef DEBUG
+  printf("explode rock: %3ld %9.2e %9.2e %9.2e %9.2e %04X %04X %9.2e %9.2e %04x %04X\n", rockp - rock, rockp->x, rockp->y, rockp->xvel, rockp->yvel, rockp->type_class, rockp->type_number, rockp->heat, rockp->heat_capacity, rockp->greeb, rockp->plume_angle);
+#endif
+}
 
 void display_rocks(SDL_Surface *surf_screen) {
   int i;
@@ -133,43 +186,7 @@ void display_rocks(SDL_Surface *surf_screen) {
       // the rock flashes to steam, releasing enough energy to destroy
       // the rock in spectacular fashion.
       if(rcki->heat > image->h) {
-	int g;
-	short x, y;
-
-	rcki->active = 0;
-	play_sound(1 + (int) (rnd() * 3));
-	switch(rcki->type_class) {
-	case 2: //Ice rocks are different
-	  x = rcki->x + image->w / 2;
-	  y = rcki->y + image->h / 2;
-	  create_plumedots(1659, x, y, 180, 360, 10.9312);
-	  create_plumedots(539, x, y, 180, 360, 2.73002);
-	  create_plumedots(367, x, y, 90, 109, 4.656);
-	  create_plumedots(367, x, y, 270, 109, 4.656);
-	  create_plumedots(467, x + rcki->xvel, y + rcki->yvel, 90, 109, 1.7656);
-	  create_plumedots(467, x + rcki->xvel, y + rcki->yvel, 270, 109, 1.7656);
-	  create_plumedots(426, x - rcki->xvel, y - rcki->yvel, 90, 109, 1.37656);
-	  create_plumedots(426, x - rcki->xvel, y - rcki->yvel, 270, 109, 1.37656);
-	  break;
-	default:
-	  makebangdots(rcki->x, rcki->y, rcki->xvel, rcki->yvel,
-		       image, 10, 3);
-	}
-
-	// If a greeblie was going to this rock
-	if((g = rcki->greeb)) {
-	  // ...and had landed, then the greeblie explodes too.
-	  if(the_greeblies[g].landed)
-	    kill_greeb(g);
-	  else {
-	    // If the greeblie is not yet landed, then it must now
-	    // find another rock to jump to. Choose the last active
-	    // rock found.
-	    rcki->greeb = 0;
-	    rock[last_rock].greeb = g;
-	    the_greeblies[g].target_rock_number = last_rock;
-	  }
-	}
+	way_of_the_exploding_rock(rcki, last_rock, image, surf_screen);
       }
     }
     // Draw any greeblies attached to (or going to) the rock, whether the rock is active or not.
@@ -202,22 +219,30 @@ void *init_rocks(void) {
       guru_meditation(GM_FLAGS_RECOVERY | GM_FLAGS_ABORTIFY | GM_FLAGS_CHOICE, GM_SS_BootStrap | GM_GE_BadParm | GURU_SEC_rocks, &init_rocks);
     }
   }
-  if((surf_rocks = load_images_ck("rock%hd.bmp", 0, 255, 0)) == NULL)
+  if((surf_rocks = load_images_ck("rock%hd.bmp", 0, 255, 0)) == NULL) {
     guru_meditation(GM_FLAGS_DEADEND, GM_SS_Graphics | GM_GE_IOError | GURU_SEC_rocks, (void*)0x524f434b);
-  if((surf_dead_rocks = load_images_ck("deadrock%hd.bmp", 0, 255, 0)) == NULL)
+  }
+  if((surf_dead_rocks = load_images_ck("deadrock%hd.bmp", 0, 255, 0)) == NULL) {
     guru_meditation(GM_FLAGS_DEADEND, GM_SS_Graphics | GM_GE_IOError | GURU_SEC_rocks, (void*)~0x524f434b);
-  if((surf_lithiumrocks = load_images_ck("lithiumrock.%02hX.ilbm", 0, 255, 0)) == NULL)
+  }
+  if((surf_lithiumrocks = load_images_ck("lithiumrock.%02hX.png", 0, 255, 0)) == NULL) {
     guru_meditation(GM_FLAGS_DEADEND, GM_SS_Graphics | GM_GE_IOError | GURU_SEC_rocks, (void*)0x4c495448);
-  if((surf_dead_lithiumrocks = load_images_ck("deadlithiumrock.%02hX.ilbm", 0, 255, 0)) == NULL)
+  }
+  if((surf_dead_lithiumrocks = load_images_ck("deadlithiumrock.%02hX.png", 0, 255, 0)) == NULL) {
     guru_meditation(GM_FLAGS_DEADEND, GM_SS_Graphics | GM_GE_IOError | GURU_SEC_rocks, (void*)~0x4c495448);
-  if((surf_icerocks = load_images_ck("icerock.%02hX.png", 0, 255, 0)) == NULL)
+  }
+  if((surf_icerocks = load_images_ck("icerock.%02hX.png", 0, 255, 0)) == NULL) {
     guru_meditation(GM_FLAGS_DEADEND, GM_SS_Graphics | GM_GE_IOError | GURU_SEC_rocks, (void*)0x494345);
-  if((surf_dead_icerocks = load_images_ck("deadicerock.%02hX.png", 0, 255, 0)) == NULL)
+  }
+  if((surf_dead_icerocks = load_images_ck("deadicerock.%02hX.png", 0, 255, 0)) == NULL) {
     guru_meditation(GM_FLAGS_DEADEND, GM_SS_Graphics | GM_GE_IOError | GURU_SEC_rocks, (void*)~0x494345);
+  }
   if(!surf_rocks || !surf_dead_rocks ||
      !surf_lithiumrocks || !surf_dead_lithiumrocks ||
      !surf_icerocks || !surf_dead_icerocks
-     ) return NULL;
+     ) {
+    return NULL;
+  }
   for(ptrs = surfs; *ptrs != NULL; ++ptrs) {
     assert((*ptrs)->num_surfaces == (*(ptrs + 1))->num_surfaces);
     if(!((*ptrs)->num_surfaces == (*(ptrs + 1))->num_surfaces))
@@ -231,3 +256,33 @@ void *init_rocks(void) {
   }
   return surf_rocks;
 }
+
+
+void reset_rocks(void) {
+  unsigned int i;
+
+  rockspeed = 5.0 * xsize / 640.0;
+  for(i = 0; i < MAX_ROCKS; i++)
+    rock[i].active = 0;
+  rockrate = 0.0;
+  countdown = 1.111;
+}
+
+
+void ingame_maybe_rock() {
+  if(state != GAME_PAUSED) {
+    rockspeed = 4.885 * xsize / 640.0 + sqrtf(floorf(level)) / 5.0;
+    rockrate = 0.13182 + sqrtf(floorf(level)) / 18.0;
+    if(level - (int) level < 0.8) {
+      if(state != GAME_PLAY && state != DEMO) {
+	countdown -= 0.3 * movementrate;
+      } else {
+	countdown -= rockrate * movementrate;
+      }
+      while(countdown < 0) {
+	countdown++;
+	create_rock();
+      }
+    }
+  }
+}
