@@ -21,6 +21,8 @@
  * ----------
  *
  * uint32	maximum number of dots/sparkles
+ * uint16	further flags for sparkles engine (= 0)
+ *		  (bit 0 = 1: debug output on overflow)
  */
 
 #ifdef DEBUG
@@ -33,6 +35,7 @@ sparkle_life_colours_t cool_colours;
 static sparkle_t *sparkles;
 static sparkle_t *sparkles_end;
 uint32_t max_bang_dots = MAX_BANG_DOTS;
+uint16_t mdot_flags = 0;
 
 sparkle_t *init_sparkles(uiff_ctx_t iff) {
   int32_t size;
@@ -50,6 +53,11 @@ sparkle_t *init_sparkles(uiff_ctx_t iff) {
       max_bang_dots = read32(iff.f);
       assert(printf("FORM.ROCK FORM.SPRK MDOT size = $%08X mdot = $%08X\n", size, max_bang_dots));
     }
+    if(size >= 6) {
+      //Chunk found and it is long enough...
+      mdot_flags = read16(iff.f);
+      assert(printf("FORM.ROCK FORM.SPRK MDOT size = $%08X mdot_flags = $%04hX\n", size, mdot_flags));
+    }
     uiff_rewind_group(&iff);
     index = 0;
     while((size = uiff_find_chunk_ctx(&iff, MakeID('C', 'M', 'A', 'P'))) > 0) {
@@ -154,12 +162,17 @@ sparkle_t *create_sparkle(float x, float y, float dx, float dy, float decay, spa
     max_bangdots_used = nmax;
     printf("new: max_bangdots_used = $%lX\n", max_bangdots_used);
   }
+  //In debug mode always guru meditate.
+  mdot_flags |= 1;
 #endif
   assert(hot_colours.clist != NULL);
   assert(cool_colours.clist != NULL);
   if(sparkles_end - sparkles >= max_bang_dots) {
-    guru_meditation(GM_FLAGS_GREEN, GM_SS_Intuition | GM_GE_ProcCreate | GURU_SEC_bangdots, &create_sparkle);
+    if((mdot_flags & 0x0001) != 0) {
+      guru_meditation(GM_FLAGS_GREEN, GM_SS_Intuition | GM_GE_ProcCreate | GURU_SEC_bangdots, &create_sparkle);
+    }
   } else {
+    assert(sparkles_end < sparkles + max_bang_dots);
     sparkles_end->x = x;
     sparkles_end->y = y;
     sparkles_end->dx = dx;
@@ -298,6 +311,11 @@ void create_plumedots(int num, short x, short y, short heading, short angle, flo
   float fheading;
   float speed_;
 
+  /*
+   * The -sinf is needed as the Y coordinate increases from top to
+   * bottom and we would like to have the angles in the mathematical
+   * sense. So 0 is to the right, 90 up, 180 left and 270 down.
+   */
   while(--num > 0 && sparkles_end - sparkles < max_bang_dots) {
     fheading = heading + (rnd() - 0.5) * angle;
     fheading *= 2 * M_PI / 360.0;
